"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApolloServer = void 0;
const cors_1 = __importDefault(require("@koa/cors"));
const koa_bodyparser_1 = __importDefault(require("koa-bodyparser"));
const koa_compose_1 = __importDefault(require("koa-compose"));
const graphql_playground_html_1 = require("@apollographql/graphql-playground-html");
const apollo_server_core_1 = require("apollo-server-core");
const accepts_1 = __importDefault(require("accepts"));
const koaApollo_1 = require("./koaApollo");
const middlewareFromPath = (path, middleware) => (ctx, next) => {
    if (ctx.path === path) {
        return middleware(ctx, next);
    }
    else {
        return next();
    }
};
class ApolloServer extends apollo_server_core_1.ApolloServerBase {
    async createGraphQLServerOptions(ctx) {
        return super.graphQLServerOptions({ ctx });
    }
    applyMiddleware({ app, ...rest }) {
        app.use(this.getMiddleware(rest));
    }
    getMiddleware({ path, cors, bodyParserConfig, disableHealthCheck, onHealthCheck, } = {}) {
        if (!path)
            path = '/graphql';
        const promiseWillStart = this.willStart();
        const middlewares = [];
        middlewares.push(middlewareFromPath(path, async (_ctx, next) => {
            await promiseWillStart;
            return next();
        }));
        if (!disableHealthCheck) {
            middlewares.push(middlewareFromPath('/.well-known/apollo/server-health', (ctx) => {
                ctx.set('Content-Type', 'application/health+json');
                if (onHealthCheck) {
                    return onHealthCheck(ctx)
                        .then(() => {
                        ctx.body = { status: 'pass' };
                    })
                        .catch(() => {
                        ctx.status = 503;
                        ctx.body = { status: 'fail' };
                    });
                }
                else {
                    ctx.body = { status: 'pass' };
                }
            }));
        }
        this.graphqlPath = path;
        if (cors === true) {
            middlewares.push(middlewareFromPath(path, cors_1.default()));
        }
        else if (cors !== false) {
            middlewares.push(middlewareFromPath(path, cors_1.default(cors)));
        }
        if (bodyParserConfig === true) {
            middlewares.push(middlewareFromPath(path, koa_bodyparser_1.default()));
        }
        else if (bodyParserConfig !== false) {
            middlewares.push(middlewareFromPath(path, koa_bodyparser_1.default(bodyParserConfig)));
        }
        middlewares.push(middlewareFromPath(path, (ctx, next) => {
            if (ctx.request.method === 'OPTIONS') {
                ctx.status = 204;
                ctx.body = '';
                return;
            }
            if (this.playgroundOptions && ctx.request.method === 'GET') {
                const accept = accepts_1.default(ctx.req);
                const types = accept.types();
                const prefersHTML = types.find((x) => x === 'text/html' || x === 'application/json') === 'text/html';
                if (prefersHTML) {
                    const playgroundRenderPageOptions = {
                        endpoint: path,
                        ...this.playgroundOptions,
                    };
                    ctx.set('Content-Type', 'text/html');
                    const playground = graphql_playground_html_1.renderPlaygroundPage(playgroundRenderPageOptions);
                    ctx.body = playground;
                    return;
                }
            }
            return koaApollo_1.graphqlKoa(() => {
                return this.createGraphQLServerOptions(ctx);
            })(ctx, next);
        }));
        return koa_compose_1.default(middlewares);
    }
}
exports.ApolloServer = ApolloServer;
//# sourceMappingURL=ApolloServer.js.map